"""ETI/Domo analog sensor device."""

import logging
from typing import Optional

from .base import TYPE_ANALOG_SENSOR, CameDevice, DeviceState, StateType

_LOGGER = logging.getLogger(__name__)


class CameAnalogSensor(CameDevice):
    """ETI/Domo analog sensor device class."""

    def __init__(
        self,
        manager,
        device_info: DeviceState,
        update_cmd_base: str = "thermo",
        update_src_field: str = "array",
        device_class: Optional[str] = None,
    ):
        """Init instance."""
        super().__init__(
            manager, TYPE_ANALOG_SENSOR, device_info, device_class=device_class
        )

        self._update_cmd_base = update_cmd_base
        self._update_src_field = update_src_field

    def update(self):
        """Update device state."""
        self._force_update(self._update_cmd_base, self._update_src_field)

    @property
    def state(self) -> StateType:
        """Return the current device state."""
        return self._device_info.get("value")

    @property
    def unit_of_measurement(self) -> Optional[str]:
        """Return the unit of measurement of this sensor, if any."""
        return self._device_info.get("unit")



"""ETI/Domo light device."""

import colorsys
import logging
from typing import List

from .base import TYPE_LIGHT, CameDevice, DeviceState

_LOGGER = logging.getLogger(__name__)

# Light types
LIGHT_TYPE_STEP_STEP = "STEP_STEP"
LIGHT_TYPE_DIMMER = "DIMMER"
LIGHT_TYPE_RGB = "RGB"

# Light states
LIGHT_STATE_OFF = 0
LIGHT_STATE_ON = 1
LIGHT_STATE_AUTO = 4


class CameLight(CameDevice):
    """ETI/Domo light device class."""

    def __init__(self, manager, device_info: DeviceState):
        """Init instance."""
        super().__init__(manager, TYPE_LIGHT, device_info)

    @property
    def light_type(self) -> str:
        """Get light type."""
        return self._device_info.get("type")

    @property
    def support_color(self) -> bool:
        """Return True if light support color as HS values."""
        # pylint: disable=fixme
        # todo: Which type is correct? "RGB" or "rgb"
        return self.light_type.upper() == LIGHT_TYPE_RGB

    @property
    def rgb_color(self) -> List[int]:
        """Return the RGB color of the light."""
        perc = int(self._device_info.get("perc", 100) * 255 / 100)
        return self._device_info.get("rgb", [perc, perc, perc])

    @property
    def _hsv_color(self) -> List[int]:
        """Return the HSV color of the light."""
        rgb = self.rgb_color
        hsv = colorsys.rgb_to_hsv(rgb[0], rgb[1], rgb[2])
        return [round(hsv[0] * 360), round(hsv[1] * 100), round(hsv[2] * 100 / 255)]

    @property
    def hs_color(self) -> List[int]:
        """Return the HS color of the light."""
        return self._hsv_color[0:2]

    def set_rgb_color(self, rgb: List[int]):
        """Set RGB color of light."""
        if not self.support_color:
            return

        if rgb[0] < 0:
            rgb[0] = 0
        elif rgb[0] > 255:
            rgb[0] = 255
        if rgb[1] < 0:
            rgb[1] = 0
        elif rgb[1] > 255:
            rgb[1] = 255
        if rgb[2] < 0:
            rgb[2] = 0
        elif rgb[2] > 255:
            rgb[2] = 255

        self.switch(rgb=rgb)

    def set_hs_color(self, hs: List[float]):
        """Set HS color of light."""
        if not self.support_color:
            return

        if hs[0] < 0:
            hs[0] = 0
        elif hs[0] > 360:
            hs[0] = 360
        if hs[1] < 0:
            hs[1] = 0
        elif hs[1] > 100:
            hs[1] = 100

        if self.support_color:
            hsv = self._hsv_color
            self.switch(
                rgb=list(
                    map(
                        int,
                        colorsys.hsv_to_rgb(
                            hs[0] / 360, hs[1] / 100, hsv[2] * 255 / 100
                        ),
                    )
                )
            )

    @property
    def support_brightness(self) -> bool:
        """Return True if light support brightness in percents."""
        return self.light_type in (LIGHT_TYPE_DIMMER, LIGHT_TYPE_RGB)

    @property
    def brightness(self) -> int:
        """Get light brightness in percents."""
        if self.support_color:
            return self._hsv_color[2]

        return self._device_info.get("perc", 100)  # Applicable only for dimmers

    def set_brightness(self, brightness: int):
        """Set light brightness in percents."""
        if not self.support_brightness:
            return

        if brightness < 0:
            brightness = 0
        elif brightness > 100:
            brightness = 100

        if self.support_color:
            hsv = self._hsv_color
            self.switch(
                rgb=list(
                    map(
                        int,
                        colorsys.hsv_to_rgb(
                            hsv[0] / 360, hsv[1] / 100, brightness * 255 / 100
                        ),
                    )
                )
            )

        else:
            self.switch(brightness=brightness)

    def switch(self, state: int = None, brightness: int = None, rgb: List[int] = None):
        """Switch light to new state."""
        if state is None and brightness is None and rgb is None:
            raise ValueError("At least one parameter is required")

        self._check_act_id()

        cmd = {
            "cmd_name": "light_switch_req",
            "act_id": self.act_id,
            "wanted_status": state if state is not None else self.state,
        }
        log = {}
        if state is not None:
            log["status"] = cmd["wanted_status"]
        if brightness is not None:
            log["perc"] = cmd["perc"] = brightness
        if rgb is not None:
            log["rgb"] = cmd["rgb"] = rgb[0:3]

        _LOGGER.debug('Set new state for light "%s": %s', self.name, log)

        self._manager.application_request(cmd)

    def turn_off(self):
        """Turn off light."""
        self.switch(LIGHT_STATE_OFF)

    def turn_on(self):
        """Turn on light."""
        self.switch(LIGHT_STATE_ON)

    def turn_auto(self):
        """Switch light to automatic mode."""
        self.switch(LIGHT_STATE_AUTO)

    def update(self):
        """Update device state."""
        self._force_update("light")



"""Python client models for ETI/Domo."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Dict


@dataclass
class Floor:
    """Object holding the ETI/Domo floor description."""

    id: int
    name: str

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> Floor:
        """Return a State object from a ETI/Domo API response."""

        return Floor(
            id=int(data["floor_ind"]),
            name=str(data["name"]),
        )


@dataclass
class Room:
    """Object holding the ETI/Domo room description."""

    id: int
    name: str
    floor_id: int

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> Room:
        """Return a State object from a ETI/Domo API response."""

        return Room(
            id=int(data["room_ind"]),
            name=str(data["name"]),
            floor_id=int(data["floor_ind"]),
        )



"""ETI/Domo devices subpackage."""

import logging
from typing import List

from pycame.devices.came_analog_sensor import CameAnalogSensor

from .base import CameDevice
from .came_light import CameLight
from .came_thermo import CameThermo

_LOGGER = logging.getLogger(__name__)


def get_featured_devices(manager, feature: str) -> List[CameDevice]:
    """Get device implementations for given feature."""
    devices = []

    if feature == "lights":
        cmd = {
            "cmd_name": "light_list_req",
            "topologic_scope": "plant",
        }
        response = manager.application_request(cmd, "light_list_resp")

        for device_info in response.get("array", []):
            devices.append(CameLight(manager, device_info))

        return devices

    if feature == "thermoregulation":
        cmd = {
            "cmd_name": "thermo_list_req",
            "topologic_scope": "plant",
        }
        response = manager.application_request(cmd, "thermo_list_resp")

        for device_info in response.get("array", []):
            devices.append(CameThermo(manager, device_info))

        for sensor in ["temperature", "humidity", "pressure"]:
            res = response.get(sensor)
            if res is not None:
                devices.append(
                    CameAnalogSensor(
                        manager, res, "thermo", sensor, device_class=sensor
                    )
                )

        return devices

    _LOGGER.warning("Unsupported feature type: %s", feature)
    return []



"""ETI/Domo abstract devices."""

import logging
from abc import ABC, abstractmethod
from typing import Any, Dict, Optional, Union

from _sha1 import sha1

from ..exceptions import ETIDomoUnmanagedDeviceError
from ..models import Floor, Room

_LOGGER = logging.getLogger(__name__)


TYPE_LIGHT = 0
TYPE_THERMOSTAT = 2
TYPE_ANALOG_SENSOR = -1

TYPES = {
    # Internal types
    -1: "Analog Sensor",
    # CAME types
    0: "Light",
    1: "Opening",
    2: "Thermostat",
    3: "Page",
    4: "Scenario",
    5: "Camera",
    6: "Security Panel",
    7: "Security Area",
    8: "Security Scenario",
    9: "Security Input",
    10: "Security output",
    11: "Generic relay",
    12: "Generic text",  # currently disabled
    13: "Sound zone",
    14: "Digital input",  # technical alarm
}

StateType = Union[None, str, int, float]
DeviceState = Dict[str, Any]


class CameDevice(ABC):
    """ETI/Domo abstract device class."""

    @abstractmethod
    def __init__(
        self,
        manager,
        type_id: int,
        device_info: DeviceState,
        device_class: Optional[str] = "",
    ):
        """Init instance."""
        self._manager = manager
        self._type_id = type_id
        self._device_info = device_info

        self._device_class = device_class if device_class != "" else self.type.lower()

    @property
    def unique_id(self) -> str:
        """Return the unique ID of device."""
        return "-".join(
            [str(self._type_id), sha1(self.name.encode("utf-8")).hexdigest()]
        )

    @property
    def type_id(self) -> int:
        """Return the type ID of device."""
        return self._type_id

    @property
    def type(self) -> str:
        """Return the type of device."""
        return TYPES[self._type_id]

    @property
    def name(self) -> Optional[str]:
        """Return the name of device."""
        return self._device_info.get("name")

    @property
    def act_id(self) -> Optional[int]:
        """Return the action ID for device."""
        return self._device_info.get("act_id")

    def _check_act_id(self):
        """Check for act ID availability."""
        if not self.act_id:
            raise ETIDomoUnmanagedDeviceError()

    @property
    def floor_id(self) -> Optional[int]:
        """Return the device's floor ID."""
        return self._device_info.get("floor_ind")

    @property
    def floor(self) -> Optional[Floor]:
        """Return the device's floor instance."""
        for floor in self._manager.get_all_floors():
            if floor.id == self.floor_id:
                return floor

        return Floor(id=self.floor_id, name=f"Floor #{self.floor_id}")

    @property
    def room_id(self) -> Optional[int]:
        """Return the device's room ID."""
        return self._device_info.get("room_ind")

    @property
    def room(self) -> Optional[Room]:
        """Return the device's room instance."""
        for room in self._manager.get_all_rooms():
            if room.id == self.room_id:
                return room

        return Room(
            id=self.room_id, name=f"Room #{self.room_id}", floor_id=self.floor_id
        )

    @property
    def available(self) -> bool:
        """Return True if device is available."""
        return self._manager.connected

    @property
    def state(self) -> StateType:
        """Return the current device state."""
        return self._device_info.get("status")

    def update_state(self, state: DeviceState) -> bool:
        """Update device state."""
        if state.get("act_id") != self.act_id:
            return False

        if state.get("cmd_name"):
            state.pop("cmd_name")

        log = {}
        for k, val in state.items():
            if self._device_info.get(k) != val:
                log[k] = val
        if log:
            _LOGGER.debug(
                'Received new state for %s "%s": %s',
                self.type.lower(),
                self.name,
                log,
            )

        self._device_info = state

        return bool(log)

    def _force_update(self, cmd_base: str, field: str = "array"):
        """Force update device state."""
        self._check_act_id()

        cmd = {
            "cmd_name": f"{cmd_base}_list_req",
            "topologic_scope": "act",
            "value": self.act_id,
        }
        res = self._manager.application_request(cmd, f"{cmd_base}_list_resp").get(
            field, []
        )
        if not isinstance(res, list):
            res = [res]
        for device_info in res:  # type: DeviceState
            if device_info.get("act_id") == self.act_id:
                self.update_state(device_info)
                return

    @abstractmethod
    def update(self):
        """Update device state."""
        raise NotImplementedError

    @property
    def device_class(self) -> Optional[str]:
        """Return the class of this device."""
        return self._device_class



"""ETI/Domo thermoregulation device."""

import logging
from typing import Optional

from .base import TYPE_THERMOSTAT, CameDevice, DeviceState

_LOGGER = logging.getLogger(__name__)


# Thermoregulation devices states
THERMO_STATE_OFF = 0
THERMO_STATE_ON = 1

# Thermoregulation devices modes
THERMO_MODE_OFF = 0
THERMO_MODE_MANUAL = 1
THERMO_MODE_AUTO = 2
THERMO_MODE_JOLLY = 3

# Thermoregulation devices seasons
THERMO_SEASON_OFF = "plant_off"
THERMO_SEASON_WINTER = "winter"
THERMO_SEASON_SUMMER = "summer"

# Thermoregulation devices dehumidifier states
THERMO_DEHUMIDIFIER_OFF = 0
THERMO_DEHUMIDIFIER_ON = 1

# Thermoregulation devices fan speeds
THERMO_FAN_SPEED_OFF = 0
THERMO_FAN_SPEED_SLOW = 1
THERMO_FAN_SPEED_MEDIUM = 2
THERMO_FAN_SPEED_FAST = 3
THERMO_FAN_SPEED_AUTO = 4


class CameThermo(CameDevice):
    """ETI/Domo thermoregulation device class."""

    def __init__(self, manager, device_info: DeviceState):
        """Init instance."""
        super().__init__(manager, TYPE_THERMOSTAT, device_info)

    @property
    def mode(self) -> Optional[int]:
        """Get current mode."""
        return self._device_info.get("mode")

    @property
    def season(self) -> Optional[str]:
        """Get current season mode."""
        return self._device_info.get("season")

    @property
    def current_temperature(self) -> Optional[float]:
        """Return the current temperature."""
        temp = self._device_info.get("temp", self._device_info.get("temp_dec"))
        return temp / 10 if temp is not None else None

    @property
    def target_temperature(self) -> Optional[float]:
        """Return the temperature we try to reach."""
        temp = self._device_info.get("set_point")
        return temp / 10 if temp is not None else None

    @property
    def support_target_temperature(self) -> bool:
        """Return True if device can change target temperature."""
        return True

    @property
    def dehumidifier_state(self) -> Optional[int]:
        """Return the state of dehumidifier."""
        dehumidifier = self._device_info.get("dehumidifier", {})
        return dehumidifier.get("enabled")

    @property
    def target_humidity(self) -> Optional[int]:
        """Return the humidity we try to reach."""
        dehumidifier = self._device_info.get("dehumidifier", {})
        return dehumidifier.get("setpoint")

    @property
    def support_target_humidity(self) -> bool:
        """Return True if device can change target humidity."""
        return self.target_humidity is not None

    @property
    def fan_speed(self) -> Optional[int]:
        """Get current fan speed."""
        return self._device_info.get("fan_speed")

    @property
    def support_fan_speed(self) -> bool:
        """Return True if device can change wind speed."""
        return self.fan_speed is not None

    def update(self):
        """Update device state."""
        self._force_update("thermo")

    def zone_config(
        self,
        mode: int = None,
        temperature: float = None,
        season: str = None,
        fan_speed: int = None,
    ):
        """Change device's config."""
        if (
            mode is None
            and temperature is None
            and season is None
            and fan_speed is None
        ):
            raise ValueError("At least one parameter is required")

        self._check_act_id()

        cmd = {
            "cmd_name": "thermo_zone_config_req",
            "act_id": self.act_id,
            "mode": mode if mode is not None else self._device_info.get("mode"),
            "set_point": int(temperature * 10)
            if temperature is not None
            else self._device_info.get("set_point"),
            "extended_infos": 0,
        }
        if season is not None:
            cmd["extended_infos"] = 1
            cmd["season"] = season
        if fan_speed is not None:
            cmd["extended_infos"] = 1
            cmd["fan_speed"] = fan_speed

        self._manager.application_request(cmd)

        log = {}
        for k in ["mode", "set_point", "season", "fan_speed"]:
            if k in cmd:
                log[k] = cmd[k]

        if mode is not None:
            log["mode"] = int(cmd["mode"] != THERMO_MODE_OFF)

        _LOGGER.debug('Set new status for thermostat "%s": %s', self.name, log)

    def set_target_temperature(self, temp: float) -> None:
        """Set the temperature we try to reach."""
        self.zone_config(temperature=temp)



"""Python client for ETI/Domo."""


DEBUG_DEEP = False

# Base library constants
VERSION = "0.6.1-alpha"
ISSUE_URL = "https://github.com/lrzdeveloper/python-came-manager/issues"

STARTUP_MESSAGE = f"""
-------------------------------------------------------------------
CAME ETI/Domo API Python client
Version: {VERSION}
If you have ANY issues with this you need to open an issue here:
{ISSUE_URL}
-------------------------------------------------------------------
"""



"""Python client for ETI/Domo."""

import json
import logging
from typing import List, Optional

import requests

from pycame.const import DEBUG_DEEP, STARTUP_MESSAGE, VERSION
from pycame.devices import get_featured_devices
from pycame.devices.base import CameDevice, DeviceState
from pycame.exceptions import (
    ETIDomoConnectionError,
    ETIDomoConnectionTimeoutError,
    ETIDomoError,
)
from pycame.models import Floor, Room

_LOGGER = logging.getLogger(__name__)

_STARTUP = []


class CameManager:
    """Main class for handling connections with an ETI/Domo device."""

    def __init__(
        self,
        host: str,
        username: str,
        password: str,
        token: str,
        session: Optional[requests.Session] = None,
    ):
        """Initialize connection with the ETI/Domo."""
        if not _STARTUP:
            _LOGGER.info(STARTUP_MESSAGE)
            _STARTUP.append(True)

        if token is None or token == "":
            raise ValueError("Access token is REQUIRED.")

        _LOGGER.debug("Setup ETI/Domo API for %s@%s", username, host)

        self._host = host
        self._username = username
        self._password = password
        self._token = token
        self._session = session or requests.Session()

        self._client_id = None
        self._swver = None
        self._serial = None
        self._keycode = None
        self._features = []
        self._floors = None
        self._rooms = None
        self._devices = None

    @property
    def software_version(self) -> Optional[str]:
        """Return a software version of ETI/Domo."""
        return self._swver

    @property
    def serial(self) -> Optional[str]:
        """Return a serial number of ETI/Domo."""
        return self._serial

    @property
    def keycode(self) -> Optional[str]:
        """Return a keycode for ETI/Domo."""
        return self._keycode

    def _request(self, command: dict, resp_command: str = None) -> dict:
        """Handle a request to an ETI/Domo device."""
        url = f"http://{self._host}/domo/"
        headers = {
            "User-Agent": f"PythonCameManager/{VERSION}",
            "Accept": "application/json, text/plain, */*",
            "Content-Type": "application/x-www-form-urlencoded",
            "Authorization": f"access_token {self._token}",
        }

        try:
            if DEBUG_DEEP:
                _LOGGER.debug("Send API request: %s", command)

            response = self._session.post(
                url, data={"command": json.dumps(command)}, headers=headers
            )
            response.raise_for_status()

            if DEBUG_DEEP:
                _LOGGER.debug("Response: %s", response.text)

        except requests.exceptions.ConnectTimeout as exception:
            raise ETIDomoConnectionTimeoutError(
                "Timeout occurred while connecting to ETI/Domo device."
            ) from exception

        except (
            requests.exceptions.HTTPError,
            requests.exceptions.ConnectionError,
            requests.exceptions.BaseHTTPError,
        ) as exception:
            raise ETIDomoConnectionError(
                "Error occurred while communicating with ETI/Domo device."
            ) from exception

        try:
            resp_json = response.json()
            ack_reason = resp_json.get("sl_data_ack_reason")

            if ack_reason == 0:
                cmd_name = resp_json.get("sl_cmd")
                if resp_command is not None and cmd_name != resp_command:
                    raise ETIDomoError(
                        "Invalid server response. Expected {}. Actual {}".format(
                            repr(resp_command), repr(cmd_name)
                        )
                    )

                return resp_json

            errors = {
                1: "Invalid user.",
                3: "Too many sessions during login.",
                4: "Error occurred in JSON Syntax.",
                5: "No session layer command tag.",
                6: "Unrecognized session layer command.",
                7: "No client ID in request.",
                8: "Wrong client ID in request.",
                9: "Wrong application command.",
                10: "No reply to application command, maybe service down.",
                11: "Wrong application data.",
            }

            if ack_reason in errors:
                raise ETIDomoError(errors[ack_reason], errno=ack_reason)

            raise ETIDomoError(
                f"Unknown error (#{ack_reason}).",
                errno=ack_reason,
            )

        except ValueError as ex:
            raise ETIDomoError("Error in sl_data_ack_reason, can't find value.") from ex

    @property
    def connected(self) -> bool:
        """Return True if entity is available."""
        return self._client_id is not None

    def login(self) -> None:
        """Login function for access to ETI/Domo."""
        if self._client_id:
            return

        _LOGGER.debug("Login attempt")
        response = self._request(
            {
                "sl_cmd": "sl_registration_req",
                "sl_login": self._username,
                "sl_pwd": self._password,
            },
            "sl_registration_ack",
        )

        try:
            if response["sl_client_id"]:
                _LOGGER.debug("Successful authorization.")
                self._client_id = response.get("sl_client_id")
                self._features = []
                self._devices = None
            else:
                raise ETIDomoError("Error in sl_client_id, can't get value.")
        except KeyError as ex:
            raise ETIDomoError("Error in sl_client_id, can't find value.") from ex

    def application_request(
        self, command: dict, resp_command: str = "generic_reply"
    ) -> dict:
        """Handle a request to application layer to ETI/Domo."""
        self.login()

        if DEBUG_DEEP:
            _LOGGER.debug("Send application layer API request: %s", command)

        cmd = command.copy()

        try:
            response = self._request(
                {
                    "sl_cmd": "sl_data_req",
                    "sl_client_id": self._client_id,
                    "sl_appl_msg": cmd,
                },
            )
        except ETIDomoConnectionError as err:
            _LOGGER.debug("Server goes offline.")
            self._client_id = None
            raise err

        if resp_command is not None and response.get("cmd_name") != resp_command:
            raise ETIDomoError(
                "Invalid server response. Expected {}. Actual {}".format(
                    repr(resp_command), repr(response.get("cmd_name"))
                )
            )

        return response

    def _get_features(self) -> list:
        """Get list of available features."""
        if self._features:
            return self._features

        cmd = {
            "cmd_name": "feature_list_req",
        }
        response = self.application_request(cmd, "feature_list_resp")
        self._swver = response.get("swver")
        self._serial = response.get("serial")
        self._keycode = response.get("keycode")
        self._features = response.get("list")
        return self._features

    def get_all_floors(self) -> List[Floor]:
        """Get list of available floors."""
        if self._floors is not None:
            return self._floors

        cmd = {
            "cmd_name": "floor_list_req",
            "topologic_scope": "plant",
        }
        response = self.application_request(cmd, "floor_list_resp")
        self._floors = []
        for floor in response.get("floor_list", []):
            self._floors.append(Floor.from_dict(floor))
        return self._floors

    def get_all_rooms(self) -> List[Room]:
        """Get list of available rooms."""
        if self._rooms is not None:
            return self._rooms

        cmd = {
            "cmd_name": "room_list_req",
            "topologic_scope": "plant",
        }
        response = self.application_request(cmd, "room_list_resp")
        self._rooms = []
        for room in response.get("room_list", []):
            self._rooms.append(Room.from_dict(room))
        return self._rooms

    def _update_devices(self) -> Optional[List[CameDevice]]:
        """Update devices info."""
        if self._devices is None:
            _LOGGER.debug("Update devices info")

            devices = []
            for feature in self._get_features():
                devices.extend(get_featured_devices(self, feature))

            self._devices = devices

        else:
            _LOGGER.debug("Update devices info: Use cached data")

        return self._devices

    def get_all_devices(self) -> Optional[List[CameDevice]]:
        """Get list of all discovered devices."""
        return self._update_devices()

    def get_device_by_id(self, device_id: str) -> Optional[CameDevice]:
        """Get device by unique ID."""
        for device in self.get_all_devices():
            if device.unique_id == device_id:
                return device

        return None

    def get_device_by_act_id(self, act_id: int) -> Optional[CameDevice]:
        """Get device by device's act ID."""
        for device in self.get_all_devices():
            if device.act_id == act_id:
                return device

        return None

    def get_device_by_name(self, name: str) -> Optional[CameDevice]:
        """Get device by name."""
        for device in self.get_all_devices():
            if device.name == name:
                return device

        return None

    def get_devices_by_floor(self, floor_id: int) -> List[CameDevice]:
        """Get a list of devices on a floor."""
        devices = []
        for device in self.get_all_devices():
            if device.floor_id == floor_id:
                devices.append(device)

        return devices

    def get_devices_by_room(self, room_id: int) -> List[CameDevice]:
        """Get a list of devices in a room."""
        devices = []
        for device in self.get_all_devices():
            if device.room_id == room_id:
                devices.append(device)

        return devices

    def status_update(self, timeout: Optional[int] = None) -> bool:
        """Long polling method which read status updates."""
        if self._devices is None:
            self._update_devices()
            return True

        cmd = {
            "cmd_name": "status_update_req",
        }
        if timeout is not None:
            cmd["timeout"] = timeout
        response = self.application_request(cmd, "status_update_resp")

        updated = False

        for device_info in response.get("result", []):  # type: DeviceState
            if device_info.get("cmd_name") == "plant_update_ind":
                self._devices = None
                self._update_devices()
                return True

            act_id = device_info.get("act_id")
            if act_id:
                device = self.get_device_by_act_id(act_id)
                if device is not None:
                    updated |= device.update_state(device_info)

        return updated



import os

from setuptools import find_packages, setup

setup(
    name="pycame",
    version="0.6.1-alpha",
    description="CAME ETI/Domo API Python client",
    long_description="CAME ETI/Domo API Python client",
    long_description_content_type="text/markdown",
    author="",
    author_email="",
    url="https://github.com/lrzdeveloper/python-came-manager/issues",
    license="",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Information Technology",
        "License :: Other/Proprietary License",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development",
    ],
    packages=find_packages(exclude=["examples"]),
    python_requires=">=3.8",
    install_requires=[
        "requests~=2.26",
        "aiohttp~=3.7"
    ]
)



"""Exceptions for ETI/Domo."""
from typing import Optional


class ETIDomoError(Exception):
    """Generic ETI/Domo exception."""

    def __init__(self, status: str, errno: Optional[int] = None):
        """Initialize."""
        super().__init__(status)
        self.status = status
        self.errno = errno


class ETIDomoConnectionError(ETIDomoError):
    """ETI/Domo connection exception."""


class ETIDomoConnectionTimeoutError(ETIDomoConnectionError, TimeoutError):
    """ETI/Domo connection Timeout exception."""


class ETIDomoUnmanagedDeviceError(ETIDomoError):
    """ETI/Domo exception for unmanaged device."""

    def __init__(
        self, status: str = "This device is unmanageable", errno: Optional[int] = None
    ):
        """Initialize."""
        super().__init__(status, errno)
